package bots;

import robocode.Bullet;
import robocode.GunTurnCompleteCondition;
import robocode.HitByBulletEvent;
import robocode.HitRobotEvent;
import robocode.HitWallEvent;
import robocode.RobotDeathEvent;
import robocode.ScannedRobotEvent;
import robocode.TeamRobot;
import robocode.TurnCompleteCondition;
import utils.BotInfo;
import utils.BotInfoMap;
import utils.BotMath;

/**
 * Simple reactive bot
 * @author Francesco Baldisserri
 */
public class SimpleBot extends TeamRobot {
	private static final double RADAR_TURNING_RATE=Double.POSITIVE_INFINITY;
	private static final double SWEEP_STEP=45;
	private static final double BACKWARD_DISTANCE=0.1;
	private static final double NEAR_DIST_PERC=0.3;
	private static final double MIN_FIRE_POWER=0.1;
	private static final double DELTA_TARGETING_TIME=0.1;
	private boolean tracking;
	private BotInfo target;
	private BotInfoMap bots;
	private double direction,sweep;
	private double maxDistance;
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		bots=new BotInfoMap();
		maxDistance=BotMath.pointsDistance(0,0,getBattleFieldWidth(),getBattleFieldHeight());
		tracking=false;
		sweep=0;
		while(true){
			if(tracking){
				//radar move
				direction=(BotMath.normalRelativeAngle(target.predictBearing(getTime(),this))>0)?1:-1;
				setTurnRadarRight(BotMath.normalRelativeAngle(getHeading()-getRadarHeading()+target.getBearing(this)+sweep*direction));
				sweep+=SWEEP_STEP;
				//movement move
				double distance=target.getDistance(this);
				if(distance>NEAR_DIST_PERC*maxDistance)
					goToward(target.getX(),target.getY(),distance-NEAR_DIST_PERC*maxDistance);
				//gun move
				shot(target);
			} 
			else {
				//radar move
				setTurnRadarRight(RADAR_TURNING_RATE);
				//movement move
				setAhead(maxDistance*Math.random());
				setTurnRight(BotMath.normalRelativeAngle(360*Math.random()));
			}
			execute();
		}
	}
	/* (non-Javadoc)
	 * @see robocode.Robot#onScannedRobot(robocode.ScannedRobotEvent)
	 */
	public void onScannedRobot(ScannedRobotEvent arg0) {
		super.onScannedRobot(arg0);
		sweep=0;
		bots.updateInfo(new BotInfo(arg0,this),getTime());
		if(!tracking&&!isTeammate(arg0.getName())){
			target=bots.getInfo(arg0.getName());
			tracking=true;
		}
	}
	/* (non-Javadoc)
	 * @see modules.EmptyModule#onHitRobot(robocode.HitRobotEvent)
	 */
	public void onHitRobot(HitRobotEvent arg0) {
		super.onHitRobot(arg0);
		backward(arg0.getBearing());
	}
	/* (non-Javadoc)
	 * @see modules.EmptyModule#onHitWall(robocode.HitWallEvent)
	 */
	public void onHitWall(HitWallEvent arg0) {
		super.onHitWall(arg0);
		backward(arg0.getBearing());
		//goToward(getBattleFieldWidth()/2,getBattleFieldHeight()/2,getMaxDistance()*BACKWARD_DISTANCE);
	}
	/* (non-Javadoc)
	 * @see modules.EmptyModule#onHitByBullet(robocode.HitByBulletEvent)
	 */
	public void onHitByBullet(HitByBulletEvent arg0) {
		super.onHitByBullet(arg0);
		if(tracking&&!arg0.getName().equals(target.getName())&&!isTeammate(arg0.getName()))
			tracking=false;
		setTurnRight(BotMath.normalRelativeAngle(arg0.getBearing()+90));
		backward(arg0.getBearing());
	}
	/* (non-Javadoc)
	 * @see robocode.Robot#onRobotDeath(robocode.RobotDeathEvent)
	 */
	public void onRobotDeath(RobotDeathEvent arg0) {
		bots.removeInfo(arg0.getName());
		if(tracking&&target.getName().equals(arg0.getName()))
			tracking=false;
	}
	/**
	 * @param obstacleBearing
	 */
	private void backward(double obstacleBearing){
		if(Math.abs(obstacleBearing)>90)
			setAhead(maxDistance*BACKWARD_DISTANCE);
		else
			setAhead(-maxDistance*BACKWARD_DISTANCE);
	}
	/**
	 * Moves bot toward point (x,y) for a certain distance
	 * @param x X
	 * @param y Y 
	 * @param distance Distance
	 */
	private void goToward(double x,double y,double distance){
		setTurnRight(BotMath.getBearing(getHeading(),getX(),getY(),x,y));
		waitFor(new TurnCompleteCondition(this));
		setAhead(distance);
	}
	/**
	 * @param bot
	 * @return
	 */
	private Bullet shot(BotInfo bot){
		double power=Math.max(Math.min(maxDistance/bot.getDistance(this),3),MIN_FIRE_POWER);
		double speed=BotMath.shotSpeed(power);
		double effTime=bot.getDistance(this)/speed+getTime();
		double currTime=getTime();
		double myX=getX(),myY=getY();
		double shotTime,enemyX,enemyY,oldDelta,delta=Double.MAX_VALUE;
		do{
			oldDelta=delta;
			shotTime=effTime;
			enemyX=bot.predictX((long)shotTime);
			enemyY=bot.predictY((long)shotTime);
			effTime=BotMath.pointsDistance(myX,myY,enemyX,enemyY)/speed+currTime;
			delta=Math.abs(effTime-shotTime);
		} while((delta>DELTA_TARGETING_TIME)&&(oldDelta!=delta));
		setTurnGunRight(BotMath.normalRelativeAngle(BotMath.getAngle(myX,myY,enemyX,enemyY)-getGunHeading()));
		waitFor(new GunTurnCompleteCondition(this));
		if (getGunHeat() == 0)
			return setFireBullet(power);
		else return null;
	}
}