package utils;

import robocode.*;

/** 
 * Class that holds the reconi information regarding a bot
 * at a given point in time. This information is normally
 * gathered from scanned event's and then can be used to make
 * decisions regarding aiming, and movement. 
 */
public class BotInfo{
	private static final double BOT_SIZE=18; //bot max size 
    private String name;	// holds the bot name
    private double x,y; 	// holds the bot position
    private double energy;  // holds the bot life
    private double heading; // holds the heading angle the enemy is facing
    private double speed;   // holds the last known bot speed
    private long lastUpdate;// holds the last time this infos were updated
    private double fieldWidth,fieldHeight;	//holds battle field sizes
    
    /**
     * Default constructor for BotInfo
     * @param name Robot name
     * @param width Battlefield width
     * @param height Battlefield height
     * @param heading Robot heading
     * @param x Robot X position
     * @param y Robot Y position
     * @param life Robot remaining energy
     * @param speed Robot current speed
     * @param time Current time
     */
    public BotInfo(String name,double width,double height,double heading,double x,double y,double life,double speed,long time) {
    	this.name=name;
    	this.fieldWidth=width;
    	this.fieldHeight=height;
    	update(heading,x,y,life,speed,time);
	}
    /**
     * Bot info constructor for ScannedRobotEvent
     * @param e ScannedRobotEvent
     * @param observer Observer robot
     */
    public BotInfo(ScannedRobotEvent e,Robot observer){
    	this.name=e.getName();
    	this.fieldWidth=observer.getBattleFieldWidth();
    	this.fieldHeight=observer.getBattleFieldHeight();
    	update(e,observer);
    }
    /**
     * Updates BotInfo parameters
     * @param heading Robot heading
     * @param x Robot X position
     * @param y Robot Y position
     * @param life Robot remaining energy
     * @param speed Robot current speed
     * @param time Current time
     */
    public void update(double heading,double x,double y,double life,double speed,long time){
    	this.heading = heading;
		this.energy = life;
		this.speed = speed;
		lastUpdate = time;
		this.x=x;
        this.y=y;
    }
    /**
     * Updates BotInfo parameters basing on a ScannedRobotEvent
     * @param e ScannedRobotEvent
     * @param observer Observer robot
     */
    public void update(ScannedRobotEvent e,Robot observer){
    	double tmpX=observer.getX()+Math.sin( BotMath.degree2radians(observer.getHeading()+e.getBearing()) ) * e.getDistance();
    	double tmpY=observer.getY()+Math.cos( BotMath.degree2radians(observer.getHeading()+e.getBearing()) ) * e.getDistance();
    	update(e.getHeading(),tmpX,tmpY,e.getEnergy(),e.getVelocity(),observer.getTime());
    }
    /**
     * Predict bot position in some time in the future
     * @param when Time to be predicted
     * @return X position
     */
    public double predictX(long when){
    	return Math.min(Math.max((x+Math.sin(BotMath.degree2radians(heading))*speed*(when-lastUpdate)),BOT_SIZE),fieldWidth);
    }
    /**
     * Predict bot position in some time in the future
     * @param when Time to be predicted
     * @return Y position
     */
    public double predictY(long when){
    	return Math.min(Math.max((y+Math.cos(BotMath.degree2radians(heading))*speed*(when-lastUpdate)),BOT_SIZE),fieldHeight);
    }
    /**
     * Calculates distance from observer
     * @param observer Observer robot
     * @return Calculated distance
     */
    public double getDistance(Robot observer){
    	return BotMath.pointsDistance(observer.getX(),observer.getY(),x,y);
    }
	/**
	 * @return Returns the angle.
	 */
	public double getHeading() {
		return heading;
	}
	/**
	 * @return Returns the bearing.
	 */
	public double getBearing(Robot observer) {
		return BotMath.getBearing(observer.getHeading(),observer.getX(),observer.getY(),x,y);
	}
	/**
	 * Predicts bearing in the future
	 * @param when Time to be predicted
	 * @param observer Robot observer
	 * @return Predicted bearing
	 */
	public double predictBearing(long when,Robot observer){
		return BotMath.getBearing(observer.getHeading(),observer.getX(),observer.getY(),predictX(when),predictY(when));
	}
	/**
	 * Returns last update time
	 * @return Returns last update time
	 */
	public long getLastUpdate() {
		return lastUpdate;
	}
	/**
	 * Returns bot energy
	 * @return Returns bot energy
	 */
	public double getEnergy() {
		return energy;
	}
	/**
	 * Returns bot name
	 * @return Returns bot name
	 */
	public String getName() {
		return name;
	}
	/**
	 * Returns bot speed
	 * @return Returns bot speed
	 */
	public double getSpeed() {
		return speed;
	}
	/**
	 * Returns bot x position
	 * @return Returns bot x position
	 */
	public double getX() {
		return x;
	}
	/**
	 * Returns bot y position
	 * @return Returns bot y position
	 */
	public double getY() {
		return y;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "BotInfo"+name+"\nUpdated:"+lastUpdate+"\nX:"+x+" Y:"+y
		+"\nSpeed:"+speed+" Energy:"+energy+" Heading:"+heading;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object arg0) {
		return name.equals(((BotInfo)arg0).getName());
	}
}