package utils;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import robocode.TeamRobot;

/**
 * Map for listing bot infos
 * @author Francesco Baldisserri
 */
public class BotInfoMap {
	protected Map bots;
	private Iterator it;
	/**
	 * Default constructor for BotInfoMap
	 */
	public BotInfoMap() {
		bots=new HashMap();
	}
	/**
	 * Get info about a robot (if present)
	 * @param botName Desired info robot name
	 * @return Returns desired BotInfo if present, null otherwise
	 */
	public BotInfo getInfo(String botName){
		return (BotInfo) bots.get(botName);
	}
	/**
	 * Check if map contains info about a robot
	 * @param botName Desired bot name
	 * @return Returns true if bot is present, false otherwise.
	 */
	public boolean containsInfo(String botName){
		return bots.containsKey(botName);
	}
	/**
	 * Update info regarding a bot
	 * @param bot New BotInfo
	 * @param time BotInfo taking time
	 * @return Returns true if bot was already present, false otherwise
	 */
	public boolean updateInfo(BotInfo bot,long time){
		BotInfo tmp=(BotInfo)bots.get(bot.getName());
		if(tmp!=null){	//info already present, updating info
			tmp.update(bot.getHeading(),bot.getX(),bot.getY(),bot.getEnergy(),bot.getSpeed(),time);
			return true;
		}
		else{	//info not present, inserting info
			bots.put(bot.getName(),bot);
			return false; 
		}
	}
	/**
	 * Removes BotInfo if present
	 * @param botName BotInfo name to be removed
	 * @return Returns true if bot was present, false otherwise
	 */
	public boolean removeInfo(String botName){
		return (bots.remove(botName)!=null);
	}
	/**
	 * @return Returns a BotInfo array
	 */
	public BotInfo[] getInfoArray(){
		return (BotInfo[]) bots.values().toArray(new BotInfo[bots.size()]);
	}
	/**
	 * @return Returns a String array with robot names
	 */
	public String[] getNamesArray(){
		return (String[]) bots.keySet().toArray(new String[bots.size()]);
	}
	/**
	 * Returns nearest BotInfo according to certain criteria
	 * @param owner Owner robot
	 * @param enemy Flag to set enemy/friend searching
	 * @param maxEnergy Max energy allowed
	 * @param maxInfoAge Min info freshness
	 * @return Returns nearest BotInfo according to criteria if present, null otherwise
	 */
	public BotInfo getNearestBot(TeamRobot owner,boolean enemy,double maxEnergy,long maxInfoAge){
		BotInfo nearest=null,current;
		it=bots.values().iterator();
		while(it.hasNext()){
			current=(BotInfo)it.next();
			if((!owner.isTeammate(current.getName())==enemy)
					&&(current.getEnergy()<=maxEnergy)
					&&(owner.getTime()-current.getLastUpdate()<=maxInfoAge)){
				if(nearest==null||current.getDistance(owner)<nearest.getDistance(owner)){
					nearest=current;
				}
			}
		}
		return nearest;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		BotInfo[] tmp=this.getInfoArray();
		String toString="BotInfoMap";
		for(int count=0;count<tmp.length;count++)
			toString+="\n"+tmp[count];
		return toString;
	}
}